"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  parseForESLint: () => parseForESLint2
});
module.exports = __toCommonJS(src_exports);

// src/ts.ts
var import_path2 = __toESM(require("path"));
var import_typescript = __toESM(require("typescript"));

// src/transform/index.ts
var import_path = __toESM(require("path"));

// src/transform/vue.ts
function transformForVue(code, context) {
  if (context.current) {
    return code;
  }
  const compiler = require("vue/compiler-sfc");
  const result = compiler.parse(code);
  const compiled = compiler.compileScript(result.descriptor, {
    id: "id",
    reactivityTransform: true
  });
  return compiled.content;
}

// src/transform/svelte.ts
function transformForSvelte(code, context) {
  if (context.current) {
    return code;
  }
  const svelte2tsx = require("svelte2tsx");
  const result = svelte2tsx.svelte2tsx(code, {
    filename: context.filePath
  });
  return result.code;
}

// src/transform/astro.ts
function transformForAstro(code, context) {
  if (context.current) {
    return code;
  }
  const compiler = require("astrojs-compiler-sync");
  const result = compiler.convertToTSX(code, {
    sourcefile: context.filePath
  });
  return result.code;
}

// src/transform/index.ts
function transformExtraFile(code, context) {
  const ext = import_path.default.extname(context.filePath);
  const transform = ext === ".vue" ? transformForVue : ext === ".svelte" ? transformForSvelte : ext === ".astro" ? transformForAstro : () => code;
  try {
    return transform(code, context);
  } catch {
  }
  return code;
}

// src/ts.ts
var TSServiceManager = class {
  constructor() {
    this.tsServices = /* @__PURE__ */ new Map();
  }
  getProgram(code, options) {
    const tsconfigPath = options.project;
    const fileName = normalizeFileName(toAbsolutePath(options.filePath));
    const extraFileExtensions = [...new Set(options.extraFileExtensions)];
    let serviceList = this.tsServices.get(tsconfigPath);
    if (!serviceList) {
      serviceList = [];
      this.tsServices.set(tsconfigPath, serviceList);
    }
    let service = serviceList.find(
      (service2) => extraFileExtensions.every(
        (ext) => service2.extraFileExtensions.includes(ext)
      )
    );
    if (!service) {
      service = new TSService(tsconfigPath, extraFileExtensions);
      serviceList.unshift(service);
    }
    return service.getProgram(code, fileName);
  }
};
var TSService = class {
  constructor(tsconfigPath, extraFileExtensions) {
    this.currTarget = {
      code: "",
      filePath: ""
    };
    this.fileWatchCallbacks = /* @__PURE__ */ new Map();
    this.extraFileExtensions = extraFileExtensions;
    this.watch = this.createWatch(tsconfigPath, extraFileExtensions);
  }
  getProgram(code, filePath) {
    const lastTargetFilePath = this.currTarget.filePath;
    this.currTarget = {
      code,
      filePath
    };
    const refreshTargetPaths = [filePath, lastTargetFilePath].filter((s) => s);
    for (const targetPath of refreshTargetPaths) {
      getFileNamesIncludingVirtualTSX(
        targetPath,
        this.extraFileExtensions
      ).forEach((vFilePath) => {
        var _a;
        (_a = this.fileWatchCallbacks.get(vFilePath)) == null ? void 0 : _a();
      });
    }
    const program = this.watch.getProgram().getProgram();
    program.getTypeChecker();
    return program;
  }
  createWatch(tsconfigPath, extraFileExtensions) {
    const normalizedTsconfigPath = normalizeFileName(tsconfigPath);
    const watchCompilerHost = import_typescript.default.createWatchCompilerHost(
      tsconfigPath,
      {
        noEmit: true,
        jsx: import_typescript.default.JsxEmit.Preserve,
        allowNonTsExtensions: true
      },
      import_typescript.default.sys,
      import_typescript.default.createAbstractBuilder,
      (diagnostic) => {
        throw new Error(formatDiagnostics([diagnostic]));
      },
      () => {
      },
      void 0
    );
    const original = {
      readFile: watchCompilerHost.readFile,
      fileExists: watchCompilerHost.fileExists,
      readDirectory: watchCompilerHost.readDirectory
    };
    watchCompilerHost.readDirectory = (...args) => {
      const results = original.readDirectory.call(watchCompilerHost, ...args);
      return [
        ...new Set(
          results.map(
            (result) => toVirtualTSXlFileName(result, extraFileExtensions)
          )
        )
      ];
    };
    watchCompilerHost.readFile = (fileName, ...args) => {
      const realFileName = toRealFileName(fileName, extraFileExtensions);
      const normalized = normalizeFileName(realFileName);
      if (this.currTarget.filePath === normalized) {
        return transformExtraFile(this.currTarget.code, {
          filePath: normalized,
          current: true
        });
      }
      const code = original.readFile.call(
        watchCompilerHost,
        realFileName,
        ...args
      );
      if (!code) {
        return code;
      }
      if (normalizedTsconfigPath === normalized) {
        const configJson = import_typescript.default.parseConfigFileTextToJson(realFileName, code);
        if (!configJson.config) {
          return code;
        }
        let include = void 0;
        if (configJson.config.include) {
          include = [configJson.config.include].flat().map((s) => toVirtualTSXlFileName(s, extraFileExtensions));
        }
        return JSON.stringify({
          ...configJson.config,
          include
        });
      }
      return transformExtraFile(code, {
        filePath: normalized,
        current: false
      });
    };
    watchCompilerHost.fileExists = (fileName, ...args) => original.fileExists.call(
      watchCompilerHost,
      toRealFileName(fileName, extraFileExtensions),
      ...args
    );
    watchCompilerHost.watchFile = (fileName, callback) => {
      const normalized = normalizeFileName(fileName);
      this.fileWatchCallbacks.set(
        normalized,
        () => callback(fileName, import_typescript.default.FileWatcherEventKind.Changed)
      );
      return {
        close: () => {
          this.fileWatchCallbacks.delete(normalized);
        }
      };
    };
    watchCompilerHost.watchDirectory = () => ({
      close() {
      }
    });
    watchCompilerHost.afterProgramCreate = (program) => {
      const originalDiagnostics = program.getConfigFileParsingDiagnostics();
      const configFileDiagnostics = originalDiagnostics.filter(
        (diag) => diag.category === import_typescript.default.DiagnosticCategory.Error && diag.code !== 18003
      );
      if (configFileDiagnostics.length > 0) {
        throw new Error(formatDiagnostics(configFileDiagnostics));
      }
    };
    const watch = import_typescript.default.createWatchProgram(watchCompilerHost);
    return watch;
  }
};
function getFileNamesIncludingVirtualTSX(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(extraFileExtension)) {
      return [`${fileName}.tsx`, fileName];
    }
  }
  return [fileName];
}
function toVirtualTSXlFileName(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(extraFileExtension)) {
      return `${fileName}.tsx`;
    }
  }
  return fileName;
}
function toRealFileName(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(`${extraFileExtension}.tsx`)) {
      return fileName.slice(0, -4);
    }
  }
  return fileName;
}
function formatDiagnostics(diagnostics) {
  return import_typescript.default.formatDiagnostics(diagnostics, {
    getCanonicalFileName: (f) => f,
    getCurrentDirectory: () => process.cwd(),
    getNewLine: () => "\n"
  });
}
function normalizeFileName(fileName) {
  let normalized = import_path2.default.normalize(fileName);
  if (normalized.endsWith(import_path2.default.sep)) {
    normalized = normalized.slice(0, -1);
  }
  if (import_typescript.default.sys.useCaseSensitiveFileNames) {
    return normalized;
  }
  return normalized.toLowerCase();
}
function toAbsolutePath(filePath) {
  return import_path2.default.isAbsolute(filePath) ? filePath : import_path2.default.join(process.cwd(), filePath);
}

// src/index.ts
var tsEslintParser = __toESM(require("@typescript-eslint/parser"));
var DEFAULT_EXTRA_FILE_EXTENSIONS = [".vue", ".svelte", ".astro"];
var tsServiceManager = new TSServiceManager();
function parseForESLint2(code, options = {}) {
  if (!options.project) {
    return tsEslintParser.parseForESLint(code, options);
  }
  const extraFileExtensions = options.extraFileExtensions || DEFAULT_EXTRA_FILE_EXTENSIONS;
  const programs = [];
  for (const option of iterateOptions(options)) {
    programs.push(tsServiceManager.getProgram(code, option));
  }
  let filePath = options.filePath;
  if (filePath && extraFileExtensions.some((ext) => {
    var _a;
    return (_a = options.filePath) == null ? void 0 : _a.endsWith(ext);
  })) {
    filePath = `${filePath}.tsx`;
  }
  const parserOptions = {
    ...options,
    filePath,
    programs,
    extraFileExtensions
  };
  return tsEslintParser.parseForESLint(code, parserOptions);
}
function* iterateOptions(options) {
  if (!options) {
    throw new Error("`parserOptions` is required.");
  }
  if (!options.filePath) {
    throw new Error("`filePath` is required.");
  }
  if (!options.project) {
    throw new Error(
      "Specify `parserOptions.project`. Otherwise there is no point in using this parser."
    );
  }
  for (const project of Array.isArray(options.project) ? options.project : [options.project]) {
    yield {
      project,
      filePath: options.filePath,
      extraFileExtensions: options.extraFileExtensions || DEFAULT_EXTRA_FILE_EXTENSIONS
    };
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  parseForESLint
});
